const {ipcRenderer} = require('electron');
let promptId;
let promptOptions;

let imgPath = process.env.NODE_ENV != 'development' ?
    '../../images/' :
    '../../../front/assets/images/';
let fontPath = process.env.NODE_ENV != 'development' ?
    '../../css/' :
    '../../../front/assets/css/';
let isCheckboxEnabled = false;

window.onerror = (error) => {
    if (promptId) {
        promptError('An error has occurred on the prompt window: \n' + error);
    }
};

const promptError = (e) => {
    if (e instanceof Error) {
        e = e.message;
    }
    ipcRenderer.sendSync('prompt-error:' + promptId, e);
};

const promptSubmit = (index = 0) => {
    const dataEl = document.getElementById('data');
    let data = {index: index, value: null};

    if (promptOptions.type === 'warning') {
        data.value = isCheckboxEnabled;
    } else if (promptOptions.type === 'input') {
        data.value = dataEl.value;
    } else if (promptOptions.type === 'select') {
        if (promptOptions.selectMultiple) {
            data.value = dataEl.querySelectorAll('option[selected]').map((o) => o.getAttribute('value'));
        } else {
            data.value = dataEl.value;
        }
    }
    ipcRenderer.sendSync('prompt-post-data:' + promptId, data);
};

const toggleSelection = () => {
    isCheckboxEnabled = !isCheckboxEnabled;
    document.getElementById('checkbox-img').src = checkboxImagePath(isCheckboxEnabled);
};

const checkboxImagePath = () => {
    if (isCheckboxEnabled) {
        return imgPath + 'icon_cb_selected.svg';
    }
    return imgPath + 'icon_cb_unselected.svg';
};

// eslint-disable-next-line max-lines-per-function
ipcRenderer.on('dom-ready', () => {
    promptId = document.location.hash.replace('#', '');

    try {
        promptOptions = JSON.parse(ipcRenderer.sendSync('prompt-get-options:' + promptId));
    } catch (e) {
        return promptError(e);
    }

    document.getElementById('font-settings').href = fontPath + '_fonts.css';

    document.getElementById('title-span').textContent = promptOptions.title || 'NVIDIA SDK Manager';
    document.getElementById('title-btn-close').addEventListener('click', () => promptSubmit(-1));
    document.getElementById('label').innerHTML = promptOptions.label;
    document.getElementById('checkbox-img').src = imgPath + 'icon_cb_unselected.svg';
    document.getElementById('checkbox-img').addEventListener('click', () => toggleSelection());
    document.getElementById('close-btn-img').src = imgPath + 'icon_close_grey.svg';

    const buttonsEl = document.getElementById('buttons');
    let buttonDefault = 0;
    if (Array.isArray(promptOptions.buttons) && promptOptions.buttons.length > 0) {
        buttonDefault = (promptOptions.defaultButton < promptOptions.buttons.length) ?
            promptOptions.defaultButton : (promptOptions.buttons.length - 1);
        promptOptions.buttons.forEach((value, index) => {
            let btn = document.createElement('button');
            btn.textContent = value;
            btn.addEventListener('click', () => promptSubmit(index));

            if (index == buttonDefault) {
                btn.classList.add('default');
            }

            buttonsEl.appendChild(btn);
        });
    } else {
        let btn = document.createElement('button');
        btn.textContent = 'OK';
        btn.addEventListener('click', () => promptSubmit(0));
        btn.classList.add('default');
        buttonsEl.appendChild(btn);
    }

    const dataContainerEl = document.getElementById('data-container');
    let dataEl;
    if (promptOptions.shouldShowLogo) {
        document.getElementById('logo-img').src = imgPath + 'nvidia.png';
        document.getElementById('logo').style.display = 'inline-block';
        document.getElementById('container').style.display = 'grid';
    }
    if (promptOptions.type === 'input') {
        dataEl = document.createElement('input');
        dataEl.setAttribute('type', 'text');

        if (promptOptions.value) {
            dataEl.value = promptOptions.value;
        } else {
            dataEl.value = '';
        }

        if (promptOptions.inputAttrs && typeof(promptOptions.inputAttrs) === 'object') {
            for (let k in promptOptions.inputAttrs) {
                if (!Object.prototype.hasOwnProperty.call(promptOptions.inputAttrs, k)) continue;
                dataEl.setAttribute(k, promptOptions.inputAttrs[k]);
            }
        }
        if (promptOptions.shouldShowLogo) {
            document.getElementById('container').style.gridTemplateColumns = '20% 80%';
        }

        if (promptOptions.errorMessage !== '') {
            document.getElementById('error-message').style.display = 'inline-block';
            document.getElementById('error-message').innerHTML = promptOptions.errorMessage;
        }
    } else if (promptOptions.type === 'select') {
        dataEl = document.createElement('select');
        let optionEl;

        for (let k in promptOptions.selectOptions) {
            if (!Object.prototype.hasOwnProperty.call(promptOptions.selectOptions, k)) continue;

            optionEl = document.createElement('option');
            optionEl.setAttribute('value', k);
            optionEl.textContent = promptOptions.selectOptions[k];
            if (k === promptOptions.value) {
                optionEl.setAttribute('selected', 'selected');
            }
            dataEl.appendChild(optionEl);
        }
    } else if (promptOptions.type === 'warning') {
        document.getElementById('icon-img').src = imgPath + 'icon_msg_warning.svg';
        document.getElementById('icon').style.display = 'inline-block';
        document.getElementById('checkbox').style.display = 'inline-block';

        dataEl = document.createElement('div');
        dataEl.innerHTML = promptOptions.message;
        dataEl.style.border = 'none';
    } else if (promptOptions.type === 'error') {
        document.getElementById('icon-img').src = imgPath + 'icon_msg_error.svg';
        document.getElementById('icon').style.display = 'inline-block';

        dataEl = document.createElement('div');
        dataEl.innerHTML = promptOptions.message;
        dataEl.style.border = 'none';
    } else if (promptOptions.type === 'dialog') {
        dataEl = document.createElement('div');
        dataEl.innerHTML = promptOptions.message;
        dataEl.style.border = 'none';
    } else if (promptOptions.type === 'cron-popup-dialog') {
        document.getElementById('title-span').textContent = '';
        document.getElementById('title-bar').style.background = 'black';
        document.getElementById('container').style.background = 'black';
        document.getElementsByTagName('button')[0].classList.remove('default');
        document.getElementById('container').style.padding = '0px 20px';
        document.getElementById('label').style.color = '#76b900';
        document.getElementById('label').style.fontSize = '20px';
        document.getElementById('form').style.width = '100%';
        dataEl = document.createElement('div');
        dataEl.innerHTML = promptOptions.message;
        dataEl.style.border = 'none';
        if (promptOptions.showReleaseNote) {
            document.getElementById('release-note').style.display = 'inline-block';
            document.getElementById('release-note-link').href = promptOptions.releaseNoteLink;
        }
        if (promptOptions.shouldShowLogo) {
            document.getElementById('container').style.gridTemplateColumns = '60px 90%';
        }
        if (promptOptions.showMore) {
            let {title, message} = promptOptions.showMore;
            document.getElementById('show-more').style.display = 'block';
            let showMoreTitle = document.getElementById('show-more-title');
            showMoreTitle.innerHTML = title;
            let showMoreData = document.getElementById('show-more-data');
            showMoreData.innerHTML = message;
            let showMoreIcon = document.getElementById('show-more-icon');
            showMoreIcon.src = imgPath + 'icon_chevron_right_white.svg';
            let showMoreLink = document.getElementById('show-more-link');
            let collapsedHeight = promptOptions.height;
            let expandedHeight = collapsedHeight + showMoreData.offsetHeight;
            let expanded = false;
            showMoreLink.addEventListener('click', ()=>{
                expanded = !expanded;
                if (expanded) {
                    ipcRenderer.send('update-height:' + promptId, expandedHeight);
                    showMoreData.style.visibility = 'visible';
                    showMoreIcon.src = imgPath + 'icon_chevron_down_white.svg';
                } else {
                    ipcRenderer.send('update-height:' + promptId, collapsedHeight);
                    showMoreData.style.visibility = 'hidden';
                    showMoreIcon.src = imgPath + 'icon_chevron_right_white.svg';
                }
            });
        }
    }

    document.addEventListener('keyup', (e) => {
        e.which = e.which || e.keyCode;
        if (e.which == 13) {
            promptSubmit(0);
        }
    });

    document.addEventListener('click', (e) => {
        let target = e.target;
        if (target && target.tagName.toLowerCase() == 'a') {
            if (process.getuid() === 0) {
                target.href && !target.href.startsWith('javascript') && ipcRenderer.send('open-in-browser-window:' + promptId, target.href);
            } else {
                target.href && !target.href.startsWith('javascript') && ipcRenderer.send('open-in-shell:' + promptId, 'openExternal', target.href);
            }
            e.preventDefault();
        }
    });

    dataContainerEl.appendChild(dataEl);
    dataEl.setAttribute('id', 'data');

    dataEl.focus();
});
