// All of the Node.js APIs are available in the preload process.
// It has the same sandbox as a Chrome extension, so you can't
// modify the `window` global without using the context bridge.
// Please note that you can still access the `window` global and,
// for example, add event listeners or update DOM elements.

// TODO: enable context isolation in preload script.
// const {contextBridge, ipcRenderer} = require('electron');

// // Set up context bridge between the renderer process and the main process
// contextBridge.exposeInMainWorld('sdkmAPI',
//     {
//         // Get JS representative of the main process object.
//         mainProcessObj: () => ipcRenderer.invoke('main-process-obj'),
//     },
// );

const {ipcRenderer} = require('electron');

window.sdkmAPI = {
    // Get JS representative of the main process object.

    // From Renderer Process to Main Process.
    appAction: (method, opts) => ipcRenderer.send('app-action', method, opts),
    captureDesktop: (opts, screenshotPath, sourceName) => ipcRenderer.invoke('capture-desktop', opts, screenshotPath, sourceName),
    canDisplayHostsAppWindowSync: (minWidth, minHeight) => {
        return ipcRenderer.sendSync('can-display-app-window-sync', minWidth, minHeight);
    },
    closeAllBrowserPrompts: () => ipcRenderer.send('close-all-browser-prompts'),
    dialogAction: (method, opts) => ipcRenderer.invoke('dialog-action', method, opts),
    dialogActionSync: (method, opts) => {
        return ipcRenderer.sendSync('dialog-action-sync', method, opts);
    },
    getDisplayWorkAreaSizeSync: () => {
        return ipcRenderer.sendSync('get-display-work-area-size-sync');
    },
    getDisplayScaleFactorSync: () => {
        return ipcRenderer.sendSync('get-display-scale-factor-sync');
    },
    getOffsetInCursorScreenSync: (width, height) => {
        return ipcRenderer.sendSync('get-offset-in-cursor-screen-sync', width, height);
    },
    globalInfoSync: (prop) => {
        return ipcRenderer.sendSync('global-info', prop);
    },
    mainProcessArgvSync: () => {
        return ipcRenderer.sendSync('main-process-argv');
    },
    mainWindowAction: (method, ...args) => ipcRenderer.send('main-window-action', method, ...args),
    mainWindowActionSync: (method, ...args) => {
        return ipcRenderer.sendSync('main-window-action-sync', method, ...args);
    },
    mainWindowPropSync: (prop) => {
        return ipcRenderer.sendSync('main-window-prop-sync');
    },
    openInShell: (method, ...args) => ipcRenderer.send('open-in-shell', method, ...args),
    openInBrowserWindow: (...args) => ipcRenderer.send('open-in-browser-window', ...args),
    openExternal: (...args) => ipcRenderer.send('open-external', ...args),
    primaryDisplayWorkAreaSync: () => {
        return ipcRenderer.sendSync('primary-display-work-area');
    },
    promptNewBrowserWindow: (opts) => ipcRenderer.invoke('prompt-new-browser-window', opts),
    printInMainConsole: (method, message) => ipcRenderer.send('print-message-in-main-console', method, message),
    removeUncaughtExceptionListener: () => ipcRenderer.send('remove-uncaught-exception-listener'),
    sdkmLogFilePathSync: () => {
        return ipcRenderer.sendSync('sdkm-log-file-path');
    },

    // From Main Process to Renderer Process.
    onUncaughtException: (callback) => ipcRenderer.on('uncaught-exception', callback),
};

