/*
 * spawn-worker.js: worker process.  This implementation is tightly coupled with
 * the parent process so we don't bother doing much validation.
 */

let modAssert = require('assert');
let modChild = require('child_process');

let pending = {};

process.on('SIGINT', ()=>{
    process.kill(0);
});

process.on('message', (message) =>{
    modAssert.ok(!Object.prototype.hasOwnProperty.call(pending, message['id']));
    pending[message['id']] = {
        'started': new Date(),
        'cmd': message,
    };

    let argv = message['argv'];
    modChild.execFile(argv[0], argv.slice(1), {
        'env': message['env'],
        'encoding': 'utf8',
    }, (err, stdout, stderr) => {
        delete (pending[message['id']]);
        try {
            if (process.connected) {
                process.send({
                    'id': message['id'],
                    'stdout': stdout,
                    'stderr': stderr,
                    'code': err ? err['code'] : 0,
                    'signal': err ? err['signal'] : null,
                });
            }
        } catch (e) {
            // do nothing.
        }
    });
});
